#ifndef SINETABLE_H
#define SINETABLE_H
#include <math.h>   
#include <assert.h>   

#define DEFSAMPLERATE 48000
#define RXDECIMATEDSR (DEFSAMPLERATE / 6)
#define BITRATE 1000.0 //31.25

class CSineTable
{
public:
	enum	{COSINE_TBL_LENGTH = 256 * 4,
		COSINE_TBL_BITSIZE = 8 + 2};

	CSineTable()
	{
		static const double PI = 3.141592654;
			long i;
			double arg;
			for (i = 0; i < sizeof m_SinTab / sizeof m_SinTab [0]; i += 1)
			{
				arg = (i * PI) * 2.0 / COSINE_TBL_LENGTH;
				m_SinTab[i] = (long)((double)0x7FFF * sin(arg));
			}
	}
	
	long inline CosTblLookup(unsigned short i) const
			{
			int		Position = ((i + (1 << (COSINE_TBL_BITSIZE - 1))) >> (16 - COSINE_TBL_BITSIZE))
								& ((1 << COSINE_TBL_BITSIZE) - 1);
			short	FineCorrection = (short)(i << COSINE_TBL_BITSIZE);
			long ret = m_SinTab[(Position + (COSINE_TBL_LENGTH/4)) 
								& ((1 << COSINE_TBL_BITSIZE) - 1)];
			long Interpolation = (1608 * FineCorrection) >> 16;
			Interpolation *= m_SinTab[Position];
			Interpolation >>= 16;
			ret -= Interpolation;
			return ret;
			};

	long inline SinTblLookup(unsigned short i) const
		{
			int		Position = ((i + (1 << (COSINE_TBL_BITSIZE - 1))) >> (16 - COSINE_TBL_BITSIZE)) 
								& ((1 << COSINE_TBL_BITSIZE) - 1) ;
			short	FineCorrection = (short)(i << COSINE_TBL_BITSIZE);
			assert(Position < COSINE_TBL_LENGTH);
			long ret = m_SinTab[Position];
			long Interpolation = (1608 * FineCorrection) >> 16;
			Interpolation *= m_SinTab[(Position + (COSINE_TBL_LENGTH/4)) 
								& ((1 << COSINE_TBL_BITSIZE) - 1)];
			Interpolation >>= 16;
			ret += Interpolation;
			return ret;
		};

protected:
	long	m_SinTab[COSINE_TBL_LENGTH + (COSINE_TBL_LENGTH/4)];
};

#endif
